@@ -11,15 +11,18 @@
     Copyright 2008 Société des arts technologiques (SAT),
     http://www.sat.qc.ca/
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
+
 import os
 import sys
 import optparse
 from os import path
 
 from sphinx.util.osutil import walk
+from sphinx import __display_version__
 
 # automodule options
 if 'SPHINX_APIDOC_OPTIONS' in os.environ:
@@ -52,12 +55,12 @@ def write_file(name, text, opts):
     """Write the output file for module/package <name>."""
     fname = path.join(opts.destdir, '%s.%s' % (name, opts.suffix))
     if opts.dryrun:
-        print 'Would create file %s.' % fname
+        print('Would create file %s.' % fname)
         return
     if not opts.force and path.isfile(fname):
-        print 'File %s already exists, skipping.' % fname
+        print('File %s already exists, skipping.' % fname)
     else:
-        print 'Creating file %s.' % fname
+        print('Creating file %s.' % fname)
         f = open(fname, 'w')
         try:
             f.write(text)
@@ -85,7 +88,7 @@ def create_module_file(package, module, opts):
         text = format_heading(1, '%s module' % module)
     else:
         text = ''
-    #text += format_heading(2, ':mod:`%s` Module' % module)
+    # text += format_heading(2, ':mod:`%s` Module' % module)
     text += format_directive(module, package)
     write_file(makename(package, module), text, opts)
 
@@ -94,6 +97,10 @@ def create_package_file(root, master_package, subroot, py_files, opts, subs):
     """Build the text of the file and write the file."""
     text = format_heading(1, '%s package' % makename(master_package, subroot))
 
+    if opts.modulefirst:
+        text += format_directive(subroot, master_package)
+        text += '\n'
+
     # build a list of directories that are szvpackages (contain an INITPY file)
     subs = [sub for sub in subs if path.isfile(path.join(root, sub, INITPY))]
     # if there are some package directories, add a TOC for theses subpackages
@@ -105,8 +112,8 @@ def create_package_file(root, master_package, subroot, py_files, opts, subs):
         text += '\n'
 
     submods = [path.splitext(sub)[0] for sub in py_files
-               if not shall_skip(path.join(root, sub), opts)
-               and sub != INITPY]
+               if not shall_skip(path.join(root, sub), opts) and
+               sub != INITPY]
     if submods:
         text += format_heading(2, 'Submodules')
         if opts.separatemodules:
@@ -133,8 +140,9 @@ def create_package_file(root, master_package, subroot, py_files, opts, subs):
                 text += '\n'
         text += '\n'
 
-    text += format_heading(2, 'Module contents')
-    text += format_directive(subroot, master_package)
+    if not opts.modulefirst:
+        text += format_heading(2, 'Module contents')
+        text += format_directive(subroot, master_package)
 
     write_file(makename(master_package, subroot), text, opts)
 
@@ -165,7 +173,7 @@ def shall_skip(module, opts):
     # skip if it has a "private" name and this is selected
     filename = path.basename(module)
     if filename != '__init__.py' and filename.startswith('_') and \
-        not opts.includeprivate:
+       not opts.includeprivate:
         return True
     return False
 
@@ -204,13 +212,13 @@ def recurse_tree(rootpath, excludes, opts):
             exclude_prefixes = ('.',)
         else:
             exclude_prefixes = ('.', '_')
-        subs[:] = sorted(sub for sub in subs if not sub.startswith(exclude_prefixes)
-                         and not is_excluded(path.join(root, sub), excludes))
+        subs[:] = sorted(sub for sub in subs if not sub.startswith(exclude_prefixes) and
+                         not is_excluded(path.join(root, sub), excludes))
 
         if is_pkg:
             # we are in a package with something to document
             if subs or len(py_files) > 1 or not \
-                shall_skip(path.join(root, INITPY), opts):
+               shall_skip(path.join(root, INITPY), opts):
                 subpackage = root[len(rootpath):].lstrip(path.sep).\
                     replace(path.sep, '.')
                 create_package_file(root, root_package, subpackage,
@@ -286,6 +294,10 @@ Note: By default this script will not overwrite already created files.""")
                       help='Don\'t create headings for the module/package '
                            'packages (e.g. when the docstrings already contain '
                            'them)')
+    parser.add_option('-M', '--module-first', action='store_true',
+                      dest='modulefirst',
+                      help='Put module documentation before submodule '
+                      'documentation')
     parser.add_option('-s', '--suffix', action='store', dest='suffix',
                       help='file suffix (default: rst)', default='rst')
     parser.add_option('-F', '--full', action='store_true', dest='full',
@@ -300,9 +312,15 @@ Note: By default this script will not overwrite already created files.""")
     parser.add_option('-R', '--doc-release', action='store', dest='release',
                       help='Project release, used when --full is given, '
                       'defaults to --doc-version')
+    parser.add_option('--version', action='store_true', dest='show_version',
+                      help='Show version information and exit')
 
     (opts, args) = parser.parse_args(argv[1:])
 
+    if opts.show_version:
+        print('Sphinx (sphinx-apidoc2) %s' % __display_version__)
+        return 0
+
     if not args:
         parser.error('A package path is required.')
 
@@ -314,7 +332,7 @@ Note: By default this script will not overwrite already created files.""")
     if opts.suffix.startswith('.'):
         opts.suffix = opts.suffix[1:]
     if not path.isdir(rootpath):
-        print >>sys.stderr, '%s is not a directory.' % rootpath
+        print('%s is not a directory.' % rootpath, file=sys.stderr)
         sys.exit(1)
     if not path.isdir(opts.destdir):
         if not opts.dryrun:
@@ -345,10 +363,12 @@ Note: By default this script will not overwrite already created files.""")
             epub = True,
             ext_autodoc = True,
             ext_viewcode = True,
+            ext_todo = True,
             makefile = True,
             batchfile = True,
             mastertocmaxdepth = opts.maxdepth,
             mastertoctree = text,
+            language = 'en',
         )
         if not opts.dryrun:
             qs.generate(d, silent=True, overwrite=opts.force)
